SystemOrganization addCategory: #Spec!
SystemOrganization addCategory: #'Spec-Examples'!
SystemOrganization addCategory: #'Spec-Browsers'!
SystemOrganization addCategory: #'Spec-Tests'!

TestCase subclass: #SpecInterpreterTest
	instanceVariableNames: 'specInterpreter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Tests'!

!SpecInterpreterTest commentStamp: '<historical>' prior: 0!
A SpecInterpreterTest is a test class for testing the behavior of SpecInterpreter!

!SpecInterpreterTest methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 1/19/2012 16:28'!
setUp
	"Setting up code for SpecInterpreterTest"


	specInterpreter := SpecInterpreter new.! !

!SpecInterpreterTest methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 1/19/2012 15:57'!
tearDown
	"Tearing down code for SpecInterpreterTest"


	specInterpreter := nil.! !

!SpecInterpreterTest methodsFor: 'tests' stamp: 'BenjaminVanRyseghem 1/19/2012 17:34'!
testBuildMorphFor

	| model morph |
	model := TestingComposableModel new.
	morph := specInterpreter buildMorphFor: model.
	
	self assert: (morph model == model).
	self assert: (model morph class = PluggableListMorph).! !

!SpecInterpreterTest methodsFor: 'tests' stamp: 'BenjaminVanRyseghem 1/19/2012 17:35'!
testBuildMorphForWithSpec

	| model morph |
	model := TestingComposableModel new.
	morph := specInterpreter buildMorphFor: model withSpec: #testingSpec.
	
	self assert: (morph model == model).
	self assert: (model morph class = PluggableTextMorph).! !

!SpecInterpreterTest methodsFor: 'tests' stamp: 'BenjaminVanRyseghem 1/19/2012 17:40'!
testBuildWindowFor

	| model morph |
	model := TestingComposableModel new.
	morph := specInterpreter buildWindowFor: model.
	
	self assert: ( model morph = morph).
	self assert: (morph submorphs anySatisfy: [:e | e class = PluggableListMorph ])! !

!SpecInterpreterTest methodsFor: 'tests' stamp: 'BenjaminVanRyseghem 1/19/2012 16:34'!
testConvertRandomSymbolOfClassToInstance
	
	| symbol instance |
	symbol := #PluggableListMorph.
	
	instance := specInterpreter convertSymbolOfClassToInstance: symbol.
	self assert: (instance class name = symbol)! !

!SpecInterpreterTest methodsFor: 'tests' stamp: 'BenjaminVanRyseghem 1/19/2012 16:36'!
testConvertSymbolOfClassToInstance
	
	| association instance|
	association := specInterpreter class bindings associations first.
	
	instance := specInterpreter convertSymbolOfClassToInstance: association key.
	self assert: (instance class name = association value)! !

!SpecInterpreterTest methodsFor: 'tests' stamp: 'BenjaminVanRyseghem 1/19/2012 16:03'!
testInterpretASpecModel

	| spec model morph |
	model := ComposableModel new.
	spec := {#PluggableListMorph. #model:. #model}.
	morph := specInterpreter interpretASpec: spec model: model.
	
	self assert: (morph model == model).! !

!SpecInterpreterTest methodsFor: 'tests' stamp: 'BenjaminVanRyseghem 1/19/2012 16:28'!
testInterpretASpecModelMorphAssociation

	| spec model morph |
	model := ComposableModel new.
	spec := {#PluggableListMorph. #model:. #model}.
	morph := specInterpreter interpretASpec: spec model: model.
	
	self assert: (model morph == morph).! !

!PluggableListMorph methodsFor: '*Spec' stamp: 'BenjaminVanRyseghem 1/19/2012 18:04'!
getIndexSelector: aSelector

	getIndexSelector := aSelector! !

!PluggableListMorph methodsFor: '*Spec' stamp: 'BenjaminVanRyseghem 11/7/2011 22:19'!
initialize

	super initialize.
	self initForKeystrokes.! !

!PluggableListMorph methodsFor: '*Spec' stamp: 'BenjaminVanRyseghem 1/19/2012 18:04'!
setIndexSelector: aSelector

	setIndexSelector := aSelector! !

PanelMorph subclass: #TabContainerMorph
	instanceVariableNames: 'container tab'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Examples'!

!TabContainerMorph commentStamp: '<historical>' prior: 0!
A TabContainerMorph is a morph (which should be pluggable soon) which handle a first bar for tabs button and a container which is used to display the morph corresponding to the selected tab!

!TabContainerMorph class methodsFor: 'example' stamp: 'BenjaminVanRyseghem 1/9/2012 17:46'!
example

	| tc testMorph model model2 model3 anotherMorph anotherMorph2 associations holder |
	
	model := ClassMethodBrowser new.
	model2 := ClassMethodList new.
	model3 := ClassMethodList new.
	
	testMorph := model buildWithSpec: #internSpec.
	anotherMorph := model2 buildWithSpec: #internSpec.
	anotherMorph2 := model3 buildWithSpec: #internSpec.
	
	associations := {'Test' -> testMorph. 'Another'-> anotherMorph. 'Again' -> anotherMorph2}.
	tc := self withTabs: associations.
	holder := (Smalltalk allClasses) asValueHolder.
	model listHolder: holder .
	model2 listHolder: holder.
	model3 listHolder: holder.
	
	( tc embeddedInMorphicWindowLabeled: 'test') openInWorld.
	^ tc! !

!TabContainerMorph class methodsFor: 'instance creation' stamp: 'BenjaminVanRyseghem 1/7/2012 06:04'!
withTabs: associations

	^ self new
		addTabs: associations;
		yourself.! !

!TabContainerMorph methodsFor: 'private' stamp: 'BenjaminVanRyseghem 1/7/2012 06:09'!
addAll
	
	self
		addMorph: tab
		fullFrame: (LayoutFrame
				fractions: (0 @ 0 corner: 1 @ 0)
				offsets: (0 @ 0 corner: 0 @ 35)).
				
	self
		addMorph: container
		fullFrame: (LayoutFrame 
				fractions: (0 @ 0 corner: 1 @ 1)
				offsets: (0 @ 35 corner: 0 @ 0)).! !

!TabContainerMorph methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 1/7/2012 06:02'!
addTabs: associations

	associations do: [:each |
		self tab 
			addTab: (Text fromString: each key) 
			withAction: [ self setContent: each value ].
		container submorphs isEmpty ifTrue: [ container addMorph: each value ]]! !

!TabContainerMorph methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/7/2012 06:08'!
container

	^ container! !

!TabContainerMorph methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 1/7/2012 06:14'!
initialize
	"Initialization code for TabContainerMorph"

	super initialize.
	
	self changeProportionalLayout.
	container := PanelMorph new changeTableLayout; listDirection: #bottomToTop; color: Color white; yourself.
	
	tab := PluggableTabBarMorph on: self.
	tab width: 150.
	tab color: Color white darker darker.
	
	self addAll.! !

!TabContainerMorph methodsFor: 'private' stamp: 'BenjaminVanRyseghem 1/9/2012 18:43'!
setContent: aMorph

	container removeAllMorphs.
	container addMorph: aMorph.
	aMorph color: Color white.
	aMorph takeKeyboardFocus.! !

!TabContainerMorph methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/7/2012 06:08'!
tab

	^ tab! !

!Morph methodsFor: '*Spec' stamp: 'BenjaminVanRyseghem 11/14/2011 16:32'!
addMorphWrapper: aMorphWrapper
	
	
	aMorphWrapper addIn: self.! !

!LayoutFrame methodsFor: '*Spec' stamp: 'BenjaminVanRyseghem 1/18/2012 01:57'!
fractions: fractionsOrNil offsets: offsetsOrNil

	| fractions offsets |

	fractions := fractionsOrNil ifNil: [0@0 extent: 0@0].
	offsets := offsetsOrNil ifNil: [0@0 extent: 0@0].
	
	^self
		topFraction: fractions top offset: offsets top;
		leftFraction: fractions left offset: offsets left;
		bottomFraction: fractions bottom offset: offsets bottom;
		rightFraction: fractions right offset: offsets right
! !

Announcement subclass: #ValueChanged
	instanceVariableNames: 'newContents oldContents'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec'!

!ValueChanged commentStamp: '<historical>' prior: 0!
A ValueChanged is an announcement raised when a value of a NewValueHolder is changed!

!ValueChanged class methodsFor: 'instance creation' stamp: 'BenjaminVanRyseghem 11/28/2011 09:34'!
newContents: newContents

	^ self new
		newContents: newContents;
		yourself! !

!ValueChanged class methodsFor: 'instance creation' stamp: 'BenjaminVanRyseghem 11/28/2011 09:26'!
oldContents: oldContent newContents: newContent

	^ self new
		newContents: newContent;
		oldContents: oldContent;
		yourself! !

!ValueChanged methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/28/2011 09:27'!
newContents: anObject
	
	newContents := anObject! !

!ValueChanged methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/28/2011 09:27'!
oldContents: anObject
	
	oldContents := anObject! !

Object subclass: #ComposableModel
	instanceVariableNames: 'morph focusOrder owner'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec'!

!ComposableModel commentStamp: '<historical>' prior: 0!
ComposableModel is an abstract which represent a applicative model made to be composed with other ComposableModel!

ComposableModel subclass: #ButtonModel
	instanceVariableNames: 'actionHolder labelHolder stateHolder enabledHolder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec'!

!ButtonModel commentStamp: '<historical>' prior: 0!
A ButtonComposableModel is an applicative model which handle a basic button.

| b |
b:= ButtonComposableModel new.
b openWithSpec.
b labelHolder contents: 'Click me'.

self example!

!ButtonModel class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 1/23/2012 18:52'!
internSpec

	^ {#PluggableButtonMorph.
	    		#on:getState:action:label:menu:. 	#model. #state. #action. #label. #nil.
			#getEnabledSelector:. 				#enabled.
			#hResizing:. 						#spaceFill.
			#vResizing:. 						#spaceFill.
			#eventHandler:.		{	#EventHandler. #on:send:to:. #keyStroke.	#keyStroke:fromMorph:. #model	}}! !

!ButtonModel class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 12/15/2011 14:42'!
title

	^ 'Button'! !

!ButtonModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 1/18/2012 01:34'!
action

	self morph ifNotNil: [:m | m takeKeyboardFocus ].
	self actionHolder contents value! !

!ButtonModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 1/18/2012 01:33'!
action: aBlock

	self actionHolder contents: aBlock! !

!ButtonModel methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 12/15/2011 14:30'!
actionHolder

	^ actionHolder! !

!ButtonModel methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 12/15/2011 14:30'!
actionHolder: anObject
	
	actionHolder := anObject! !

!ButtonModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 12/15/2011 14:48'!
enabled

	^ self enabledHolder contents! !

!ButtonModel methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 12/15/2011 14:47'!
enabledHolder

	^ enabledHolder! !

!ButtonModel methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 12/15/2011 14:47'!
enabledHolder: anObject
	
	enabledHolder := anObject.
	anObject whenChangedDo: [ self changed: #enabled ].
	self changed: #enabled! !

!ButtonModel methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 1/18/2012 00:49'!
initialize
	"Initialization code for ButtonComposableModel"

	super initialize.

	self actionHolder: [] asValueHolder.
	self labelHolder: '' asValueHolder.
	self stateHolder: true asValueHolder.
	self enabledHolder: true asValueHolder.! !

!ButtonModel methodsFor: 'focus' stamp: 'BenjaminVanRyseghem 1/18/2012 01:46'!
isEventForNextFocus: aKeyboardEvent

	^ aKeyboardEvent keyString = '<right>'! !

!ButtonModel methodsFor: 'focus' stamp: 'BenjaminVanRyseghem 1/18/2012 01:46'!
isEventForPreviousFocus: aKeyboardEvent 

	^ aKeyboardEvent keyString  = '<left>'! !

!ButtonModel methodsFor: 'focus' stamp: 'BenjaminVanRyseghem 1/23/2012 19:03'!
keyStroke: anEvent fromMorph: aMorph

	anEvent keyString = '<space>'
		ifTrue: [ self action ].
		
	super keyStroke: anEvent fromMorph: aMorph! !

!ButtonModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 12/15/2011 14:28'!
label

	^ self labelHolder contents! !

!ButtonModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 1/18/2012 00:33'!
label: aString

	self labelHolder contents: aString! !

!ButtonModel methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 12/15/2011 14:30'!
labelHolder

	^ labelHolder! !

!ButtonModel methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 12/15/2011 14:40'!
labelHolder: anObject
	
	labelHolder := anObject.
	anObject whenChangedDo: [ self changed: #label ].
	self changed: #label! !

!ButtonModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 12/15/2011 14:10'!
state

	^ self stateHolder contents! !

!ButtonModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 1/18/2012 00:59'!
state: aBoolean
	
	^ self stateHolder contents: aBoolean! !

!ButtonModel methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 12/15/2011 14:30'!
stateHolder

	^ stateHolder! !

!ButtonModel methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 12/15/2011 14:45'!
stateHolder: anObject
	
	stateHolder := anObject.
	anObject whenChangedDo: [ self changed: #state ].
	self changed: #state! !

ComposableModel subclass: #ClassMethodBrowser
	instanceVariableNames: 'listModel methodModel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Examples'!

!ClassMethodBrowser commentStamp: '<historical>' prior: 0!
A ClassMethodBrowser is a simple browser using spec and reusing MethodBrowser to browse classes>>methods>>sourceCode.

	cb := ClassMethodBrowser new.
	cb openWithSpec.
	cb classes: Smalltalk allClasses.!

!ClassMethodBrowser class methodsFor: 'example' stamp: 'BenjaminVanRyseghem 1/16/2012 23:17'!
example

	| cb |
	cb := ClassMethodBrowser new.
	cb openWithSpec.
	cb classes: Smalltalk allClasses.! !

!ClassMethodBrowser class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 1/18/2012 02:04'!
internSpec

	^ { #PanelMorph.
			#changeProportionalLayout.
			#addMorphWrapper:.	{	#MorphWrapper.
											#morph:. 	{#model. #listModel. #internSpec.}.
											#fullFrame:. 	{#LayoutFrame.
																#fractions:offsets:.	 ( 0@0 corner: 0.5@0.5 ).	(0@0 corner: 0@(-15)). }}.
			#addMorphWrapper:.	{	#MorphWrapper.
											#morph:. 	{#model. #methodModel. #listModel. #internSpec.}.
											#fullFrame:. 	{#LayoutFrame.
																#fractions:offsets:.	( 0.5@0 corner: 1@0.5 ).	(0@0 corner: 0@(-15)). }}.
			#addMorphWrapper:.	{	#MorphWrapper.
											#morph:. 		{#model. #methodModel. #toolbarModel. #internSpec.}.
											#fullFrame:. 	{#LayoutFrame.
																#fractions:offsets:.	 (0@0.5 corner: 1@0.5).	(0@(-15) corner: 0@15). }}.										
			#addMorphWrapper:.	{	#MorphWrapper.
											#morph:. 	{#model. #methodModel. #textModel. #internSpec.}.
											#fullFrame:. 	{#LayoutFrame.
																#fractions:offsets:.	 (0@0.5 corner: 1@1).	(0@15 corner: 0@0). }}.
			#vResizing:. #spaceFill.
			#hResizing:. #spaceFill.}.! !

!ClassMethodBrowser class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 1/15/2012 22:14'!
internSpec2

	^ { #PanelMorph.
			#changeProportionalLayout.
			#addMorphWrapper:.	{	#MorphWrapper.
											#morph:. 	{#model. #methodModel. #listModel. #internSpec.}.
											#frame:. 	( 0@0 corner: 0.5@0.5 )}.
			#addMorphWrapper:.	{	#MorphWrapper.
											#morph:. 	{#model. #listModel. #internSpec.}.
											#frame:. 	( 0.5@0 corner: 1@0.5 )}.
			#addMorphWrapper:.	{	#MorphWrapper.
											#morph:. 	{#model. #methodModel. #textModel. #internSpec.}.
											#frame:.	( 0@0.5 corner: 1@1 )}.
			#vResizing:. #spaceFill.
			#hResizing:. #spaceFill.}.! !

!ClassMethodBrowser class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 12/14/2011 18:12'!
title

	^ 'Class Method Browser'! !

!ClassMethodBrowser methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 1/15/2012 22:53'!
classes: aList

	self listHolder contents: aList! !

!ClassMethodBrowser methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 1/17/2012 19:42'!
initialize
	"Initialization code for ClassMethodBrowser"

	super initialize.

	listModel := self instantiateA: ListComposableModel.
	methodModel := self instantiateA: MethodBrowser.
	
	self focusOrder
		add: listModel;
		add: methodModel.
	
	listModel whenSelectedItemChanges: [:selection |
		selection 
			ifNotNil: [:class | 
				methodModel methods: (class methodDict values sort: [:a :b | a selector < b selector]).
				methodModel listModel resetSelection ]].
	
	methodModel wrapWith: [:method | method selector ].! !

!ClassMethodBrowser methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/15/2012 22:35'!
listHolder

	^ listModel listHolder! !

!ClassMethodBrowser methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/15/2012 22:35'!
listHolder: anHolder
	
	listModel listHolder: anHolder! !

!ClassMethodBrowser methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 12/14/2011 18:03'!
listModel

	^ listModel! !

!ClassMethodBrowser methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 12/14/2011 18:03'!
methodModel

	^ methodModel! !

ComposableModel subclass: #ClassMethodList
	instanceVariableNames: 'listModel listModel2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Examples'!

!ClassMethodList commentStamp: '<historical>' prior: 0!
| t |
t:= ClassMethodList new.
t openWithSpec.
t listHolder contents: (Smalltalk allClasses).
self example.!

!ClassMethodList classSide methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 1/15/2012 22:13'!
internSpec

	^{ #PanelMorph.
			#changeTableLayout.
			#listDirection:. #rightToLeft.
			#addMorph:. {#model. #listModel. #internSpec.}.
			#addMorph:. {#model. #listModel2. #internSpec.}.
			#vResizing:. #spaceFill.
			#hResizing:. #spaceFill.}.! !

!ClassMethodList classSide methodsFor: 'tests' stamp: 'BenjaminVanRyseghem 1/7/2012 06:20'!
internSpec2

	^{ #PanelMorph.
			#changeProportioanleLayout.
			#addMorphWrapper:.	{	#MorphWrapper.
											#morph:.	{ 	#model. #listModel. #internSpec.	}.
											#frame:.	(	0@0 corner: 0.5@1)	}.
			#addMorphWrapper:.	{	#MorphWrapper.
											#morph:.	{ 	#model. #listModel2. #internSpec.	}.
											#frame:.	(	0.5@0 corner: 1@1)	}.
			#hResizing:. 			#spaceFill.
			#vResizing:. 			#spaceFill}! !

!ClassMethodList classSide methodsFor: 'tests' stamp: 'BenjaminVanRyseghem 11/17/2011 19:44'!
miniSpec

	^ { 	#model. #listModel. #windowSpec.	}! !

!ClassMethodList classSide methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 11/17/2011 21:56'!
title

	^ 'Class Method List'! !

!ClassMethodList classSide methodsFor: 'tests' stamp: 'BenjaminVanRyseghem 11/17/2011 21:12'!
windowSpec2

	^{ #StandardWindow.
			#addMorphWrapper:.	{	#MorphWrapper.
											#morph:.	{ 	#model. #listModel. #windowSpec.	}.
											#frame:.	(	0@0 corner: 0.5@1)	}.
			#addMorphWrapper:.	{	#MorphWrapper.
											#morph:.	{ 	#model. #listModel2. #windowSpec.	}.
											#frame:.	(	0.5@0 corner: 1@1)	}.
			#title:.					'Test'											}! !

!ClassMethodList methodsFor: 'initialize' stamp: 'BenjaminVanRyseghem 1/16/2012 00:50'!
initialize
	| listHolder2 |
	
	super initialize.

	listModel := ListComposableModel new.	
	listModel2 := ListComposableModel new.
	
	listHolder2 := ComposableValueHolder on: (listModel selectionHolder) do: [:index :selection | selection contents ifNotNil: [:c | c methodDict keys sort ]].
	
	listModel2 listHolder: listHolder2.
	
	listModel selectionHolder whenChangedDo: [ listModel2 resetSelection ].! !

!ClassMethodList methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/15/2012 22:34'!
listHolder

	^ listModel listHolder! !

!ClassMethodList methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/15/2012 22:34'!
listHolder: anObject

	listModel listHolder: anObject.! !

!ClassMethodList methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/17/2011 21:04'!
listModel

	^ listModel! !

!ClassMethodList methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/17/2011 21:04'!
listModel2

	^ listModel2! !

!ClassMethodList methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/17/2011 21:04'!
listModel2: anObject
	
	listModel2 := anObject! !

!ClassMethodList methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/17/2011 21:04'!
listModel: anObject
	
	listModel := anObject! !

!ClassMethodList methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/16/2012 00:52'!
selectionHolder

	^ listModel selectionHolder! !

!ClassMethodList methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/16/2012 00:52'!
selectionHolder: anObject
	
	listModel selectionHolder: anObject! !

ComposableModel subclass: #CodeBrowser
	instanceVariableNames: 'listHolder listModel sendersModel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Browsers'!

!CodeBrowser commentStamp: '<historical>' prior: 0!
A CodeBrowser is a simple code browser.


It's composed of a list for classes, and a method browser.!

!CodeBrowser classSide methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 1/12/2012 19:42'!
internSpec

	^ { #PanelMorph.
			#changeProportionalLayout.
			#addMorphWrapper:.	{	#MorphWrapper.
											#morph:. 	{ #model.	#listModel.		#internSpec.}.
											#frame:.	(0@0 corner: 0.5@0.5).
									}.
			#addMorphWrapper:.	{	#MorphWrapper.
											#morph:.	{ #model.	#sendersModel.	#listModel.		#internSpec.}.
											#frame:.	(0.5@0 corner: 1@0.5).
									}.
			#addMorphWrapper:.	{	#MorphWrapper.
											#morph:.	{ #model.	#sendersModel.	#textModel.	#internSpec.}.
											#frame:. 	(0@0.5 corner: 1@1).
									}}! !

!CodeBrowser classSide methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 1/12/2012 19:43'!
title

	^ 'Code Browser'! !

!CodeBrowser methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 1/12/2012 19:47'!
initialize
	"Initialization code for CodeBrowser"
	| linkHolder |

	super initialize.

	listHolder := {} asValueHolder.
	listModel := ListComposableModel new.
	sendersModel := SendersBrowser new.
	
	listModel listHolder: listHolder.
	linkHolder := ComposableValueHolder on: listModel selectionHolder do: [:index :selection |
		selection contents
			ifNil: [ {} ]
			ifNotNil: [:class | class methodDict values sort: [:a :b | a selector < b selector ]]].
	sendersModel listHolder: linkHolder.
	sendersModel listModel wrapHolder contents: [:aMethod| aMethod selector ].! !

!CodeBrowser methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/12/2012 19:40'!
listHolder

	^ listHolder! !

!CodeBrowser methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/12/2012 19:40'!
listHolder: anObject
	
	listHolder := anObject! !

!CodeBrowser methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/12/2012 19:40'!
listModel

	^ listModel! !

!CodeBrowser methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/12/2012 19:40'!
listModel: anObject
	
	listModel := anObject! !

!CodeBrowser methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/12/2012 19:40'!
sendersModel

	^ sendersModel! !

!CodeBrowser methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/12/2012 19:40'!
sendersModel: anObject
	
	sendersModel := anObject! !

!ComposableModel class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 11/17/2011 21:57'!
internSpec

	^ self subclassresponsibility! !

!ComposableModel class methodsFor: 'instance creation' stamp: 'BenjaminVanRyseghem 1/17/2012 03:20'!
owner: owner

	^ self new
		owner: owner;
		yourself! !

!ComposableModel class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 11/17/2011 22:07'!
title
	
	^ 'Untitled window'! !

!ComposableModel class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 11/17/2011 22:08'!
windowSpec

	^{ #StandardWindow.
			#addMorphWrapper:.	{	#MorphWrapper.
											#morph:.	self internSpec.
											#frame:.	(	0@0 corner: 1@1)	}.
			#title:.					self title}! !

!ComposableModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 1/19/2012 16:38'!
buildWithSpec

	^ self buildWithSpec: #internSpec! !

!ComposableModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 1/3/2012 17:29'!
buildWithSpec: aSymbol

	^ SpecInterpreter buildMorphFor: self withSpec: aSymbol! !

!ComposableModel methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/17/2012 17:40'!
focusOrder

	^ focusOrder ifNil: [ focusOrder := OrderedCollection new ].! !

!ComposableModel methodsFor: 'focus' stamp: 'BenjaminVanRyseghem 1/18/2012 02:10'!
giveFocusToNextFrom: aMorph

	| focus |
	focus := self focusOrder.

	(focus includes: aMorph)
		ifTrue: [
			| index next |
			index := (focus indexOf: aMorph) + 1.
			(index <= focus size)
				ifTrue: [
					next := focus at: index.
					^ next takeKeyboardFocus ]].

	owner 
		ifNil: [ " I loop "
			| next |
			next := focus at: 1 ifAbsent: [ ^ self ].
			next takeKeyboardFocus ]
		ifNotNil: [ owner giveFocusToNextFrom: self ]! !

!ComposableModel methodsFor: 'focus' stamp: 'BenjaminVanRyseghem 1/23/2012 19:18'!
giveFocusToPreviousFrom: aMorph 

	| focus |
	focus := self focusOrder.

	(focus includes: aMorph)
		ifTrue: [
			| index previous |
			index := (focus indexOf: aMorph) - 1.
			(index > 0)
				ifTrue: [
					previous := focus at: index.
					^ previous takeLastKeyboardFocus ]].

	owner 
		ifNil: [ " I loop "
			| next |
			next := focus at: focus size ifAbsent: [ ^ self ].
			next takeLastKeyboardFocus ]
		ifNotNil: [ owner giveFocusToPreviousFrom: self ]! !

!ComposableModel methodsFor: 'focus' stamp: 'BenjaminVanRyseghem 1/18/2012 01:31'!
handlesKeyboard: evt

	^ true! !

!ComposableModel methodsFor: 'instance creation' stamp: 'BenjaminVanRyseghem 1/17/2012 03:21'!
instantiateA: aComposableModelClass

	^ aComposableModelClass owner: self! !

!ComposableModel methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/17/2011 21:13'!
internSpec

	^ self class internSpec! !

!ComposableModel methodsFor: 'focus' stamp: 'BenjaminVanRyseghem 1/17/2012 18:40'!
isEventForNextFocus: aKeyboardEvent

	^ false! !

!ComposableModel methodsFor: 'focus' stamp: 'BenjaminVanRyseghem 1/17/2012 18:40'!
isEventForPreviousFocus: aKeyboardEvent 
	
	^ false! !

!ComposableModel methodsFor: 'focus' stamp: 'BenjaminVanRyseghem 1/23/2012 19:12'!
keyStroke: anEvent fromMorph: aMorph

	(self isEventForNextFocus: anEvent)
		ifTrue: [ ^ self giveFocusToNextFrom: aMorph ].
		
	(self isEventForPreviousFocus: anEvent)
		ifTrue: [ ^ self giveFocusToPreviousFrom: aMorph ]! !

!ComposableModel methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/7/2011 22:21'!
morph

	^ morph! !

!ComposableModel methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/7/2011 22:21'!
morph: anObject
	
	morph := anObject! !

!ComposableModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 12/21/2011 13:33'!
openMorphWithSpec

	^ self buildWithSpec openInWorld! !

!ComposableModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 1/3/2012 17:27'!
openWithSpec
	| window |

	window := StandardWindow new
					addMorph: self buildWithSpec
					frame: (0@0 corner: 1@1).
	window openInWorld.
	window title: self title.
	^ window! !

!ComposableModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 1/3/2012 17:28'!
openWithSpec: aSymbol
	| window |

	window := StandardWindow new
					addMorph: (self buildWithSpec: aSymbol)
					frame: (0@0 corner: 1@1).
	window openInWorld.
	window title: self title.
	^ window! !

!ComposableModel methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/17/2012 18:31'!
owner: anObject
	
	owner := anObject! !

!ComposableModel methodsFor: 'focus' stamp: 'BenjaminVanRyseghem 1/18/2012 02:10'!
takeKeyboardFocus

	self focusOrder ifNotEmpty:[:focus | ^ focus first takeKeyboardFocus ].
	^ self morph ifNotNil: [:m | m takeKeyboardFocus ]! !

!ComposableModel methodsFor: 'focus' stamp: 'BenjaminVanRyseghem 1/17/2012 19:51'!
takeLastKeyboardFocus

	self focusOrder
		ifEmpty: [ self takeKeyboardFocus ]
		ifNotEmpty: [:focus | focus last takeKeyboardFocus ].! !

!ComposableModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 12/21/2011 13:32'!
title

	^ self class title! !

ComposableModel subclass: #DoubleClassMethodList
	instanceVariableNames: 'classMethodList classMethodList2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Examples'!

!DoubleClassMethodList commentStamp: '<historical>' prior: 0!
| t |
t:= DoubleClassMethodList new.
t openWithSpec.
t listHolder contents: (Smalltalk allClasses).
self example!

!DoubleClassMethodList classSide methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 11/17/2011 21:58'!
internSpec

	^{ #PanelMorph.
			#changeTableLayout.
			#listDirection:. #bottomToTop.
			#addMorph:. {#model. #classMethodList. #internSpec.}.
			#addMorph:. {#model. #classMethodList2. #internSpec.}.
			#vResizing:. #spaceFill.
			#hResizing:. #spaceFill.}.! !

!DoubleClassMethodList classSide methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 11/17/2011 22:09'!
title

	^ 'Double double lists'! !

!DoubleClassMethodList methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/17/2011 20:51'!
classMethodList

	^ classMethodList! !

!DoubleClassMethodList methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/17/2011 20:51'!
classMethodList2

	^ classMethodList2! !

!DoubleClassMethodList methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/17/2011 20:51'!
classMethodList2: anObject
	
	classMethodList2 := anObject! !

!DoubleClassMethodList methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/17/2011 20:51'!
classMethodList: anObject
	
	classMethodList := anObject! !

!DoubleClassMethodList methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/16/2012 00:30'!
initialize
	| listHolder2 |

	super initialize.

	classMethodList := ClassMethodList new.
	classMethodList2 := ClassMethodList new.
	
	listHolder2 := ComposableValueHolder 
						on: (classMethodList selectionHolder)
						do: [:index :selection | 
							selection contents 
								ifNotNil: [:c | 
									Smalltalk allClasses select:[:e | e category = c category ]]].
	
	classMethodList2 listHolder: listHolder2! !

!DoubleClassMethodList methodsFor: 'entry point' stamp: 'BenjaminVanRyseghem 1/15/2012 22:32'!
listHolder

	^ classMethodList listHolder! !

!DoubleClassMethodList methodsFor: 'entry point' stamp: 'BenjaminVanRyseghem 1/15/2012 22:32'!
listHolder: aValueHolder

	classMethodList listHolder: aValueHolder! !

ComposableModel subclass: #ListComposableModel
	instanceVariableNames: 'selectionHolder listHolder wrapHolder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec'!

!ListComposableModel commentStamp: '<historical>' prior: 0!
A ListComposableModel is an applicative model which handle a basic list.

| t |
t:= ListComposableModel new.
t openWithSpec.
t listHolder contents: (Smalltalk allClasses).
self example!

!ListComposableModel class methodsFor: 'example' stamp: 'BenjaminVanRyseghem 12/15/2011 13:29'!
example
	| instance |

	instance := self new.
	instance openWithSpec.
	instance listHolder contents: (Smalltalk allClasses).! !

!ListComposableModel class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 1/23/2012 18:51'!
internSpec

	^ {#PluggableListMorph.
	    		#model:. 				#model.
			#getListSelector:. 		#getList.
			#getIndexSelector:. 	#getIndex.
			#setIndexSelector:. 	#setIndex:.
			#wrapSelector:. 		#wrapItem:.
			#hResizing:. 			#spaceFill.
			#vResizing:. 			#spaceFill.
			#eventHandler:.		{	#EventHandler. #on:send:to:. #keyStroke.	#keyStroke:fromMorph:. #model	}}! !

!ListComposableModel class methodsFor: 'instance creation' stamp: 'BenjaminVanRyseghem 12/14/2011 14:50'!
selectionHolder: indexHolder listHolder: listHolder

	^ self new
		selectionHolder: indexHolder;
		listHolder: listHolder;
		yourself! !

!ListComposableModel class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 11/17/2011 22:09'!
title

	^ 'List'! !

!ListComposableModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 12/14/2011 14:50'!
getIndex

	^ self selectionHolder index contents! !

!ListComposableModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 1/9/2012 17:47'!
getList

	^ self listHolder contents ifNil: [ {} ]! !

!ListComposableModel methodsFor: 'initialize' stamp: 'BenjaminVanRyseghem 12/14/2011 14:49'!
initialize

	super initialize.
	self selectionHolder: SelectionValueHolder new.
	self listHolder: Array new asValueHolder.
	self wrapHolder: [:object | object asStringOrText ] asValueHolder! !

!ListComposableModel methodsFor: 'focus' stamp: 'BenjaminVanRyseghem 1/17/2012 18:42'!
isEventForNextFocus: aKeyboardEvent

	^ aKeyboardEvent keyString = '<right>'! !

!ListComposableModel methodsFor: 'focus' stamp: 'BenjaminVanRyseghem 1/17/2012 19:34'!
isEventForPreviousFocus: aKeyboardEvent

	^ aKeyboardEvent keyString = '<left>'! !

!ListComposableModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 1/16/2012 00:48'!
items: aList

	listHolder contents: aList! !

!ListComposableModel methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/17/2011 17:53'!
listHolder

	^ listHolder! !

!ListComposableModel methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/28/2011 17:27'!
listHolder: anObject
	
	listHolder := anObject.
	anObject whenChangedDo: [ self changed: #getList ].
	self changed: #getList! !

!ListComposableModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 1/7/2012 06:44'!
resetSelection

	self selectionHolder reset! !

!ListComposableModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 1/18/2012 02:15'!
selectedItem

	^ self selectedItemHolder contents! !

!ListComposableModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 1/16/2012 01:49'!
selectedItemHolder

	^ self selectionHolder selection! !

!ListComposableModel methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 12/14/2011 14:49'!
selectionHolder

	^ selectionHolder! !

!ListComposableModel methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/7/2012 06:38'!
selectionHolder: anObject
	
	selectionHolder := anObject.
	anObject whenChangedDo: [ self changed: #getIndex ].
	self changed: #getIndex.! !

!ListComposableModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 12/14/2011 14:50'!
setIndex: anIndex

	self selectionHolder index contents: anIndex.
	self selectionHolder selection contents: (self getList at: anIndex ifAbsent: [ nil ]).
	self changed: #getIndex! !

!ListComposableModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 1/16/2012 00:48'!
setSelectedIndex: anIndex

	| selection |
	selection := self getList at: anIndex ifAbsent: [ ^ self ].
	selectionHolder index contents: anIndex.
	selectionHolder selection contents: selection! !

!ListComposableModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 1/16/2012 00:48'!
setSelectedItem: anItem

	| index |
	index := self getList identityIndexOf: anItem ifAbsent: [ ^ self ].
	selectionHolder index contents: index.
	selectionHolder selection contents: anItem! !

!ListComposableModel methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/28/2011 09:20'!
update: aSymbol

	aSymbol = #contents
		ifTrue:[	self changed: #getList ]! !

!ListComposableModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 1/16/2012 02:02'!
whenSelectedItemChanges: aBlock

	self selectionHolder selection whenChangedDo: aBlock! !

!ListComposableModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 1/16/2012 02:02'!
whenSelectionChanges: aBlock

	self selectionHolder whenChangedDo: aBlock! !

!ListComposableModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 1/16/2012 02:08'!
whenSelectionIndexChanges: aBlock

	self selectionHolder index whenChangedDo: aBlock! !

!ListComposableModel methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 12/14/2011 14:31'!
wrapHolder

	^ wrapHolder! !

!ListComposableModel methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 12/14/2011 14:32'!
wrapHolder: anObject
	
	wrapHolder := anObject.
	anObject whenChangedDo: [ self changed: #getList ].
	self changed: #getList! !

!ListComposableModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 12/14/2011 14:35'!
wrapItem: anObject

	^ self wrapHolder contents value: anObject! !

!ListComposableModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 1/16/2012 00:47'!
wrappingBlock: aBlock

	wrapHolder contents: aBlock! !

ComposableModel subclass: #MethodBrowser
	instanceVariableNames: 'listModel textModel toolbarModel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Examples'!

!MethodBrowser commentStamp: '<historical>' prior: 0!
A MethodBrowser is a simple browser using Spec to display a list of methods and their source code

	si := MethodBrowser new.
	si openWithSpec.
	si methods: Object methodDict values!

!MethodBrowser class methodsFor: 'example' stamp: 'BenjaminVanRyseghem 1/16/2012 23:18'!
example

	| mb |
	mb := MethodBrowser new.
	mb openWithSpec.
	mb methods: Object methodDict values! !

!MethodBrowser class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 1/18/2012 01:58'!
internSpec

	^{ #PanelMorph.
			#changeProportionalLayout.
			#addMorphWrapper:.	{	#MorphWrapper.
											#morph:. 		{#model. #listModel. #internSpec.}.
											#fullFrame:. 	{#LayoutFrame.
																#fractions:offsets:.	 (0@0 corner: 1@0.5).	(0@0 corner: 0@(-15)). }}.
			#addMorphWrapper:.	{	#MorphWrapper.
											#morph:. 		{#model. #toolbarModel. #internSpec.}.
											#fullFrame:. 	{#LayoutFrame.
																#fractions:offsets:.	 (0@0.5 corner: 1@0.5).	(0@(-15) corner: 0@15). }}.															
			#addMorphWrapper:.	{	#MorphWrapper.
											#morph:. 		{#model. #textModel. #internSpec.}.
											#fullFrame:. 	{#LayoutFrame.
																#fractions:offsets:.	 (0@0.5 corner: 1@1).	(0@15 corner: 0@0). }}.
			#vResizing:. #spaceFill.
			#hResizing:. #spaceFill.}.! !

!MethodBrowser class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 12/21/2011 15:30'!
title

	^ 'Method Browser'! !

!MethodBrowser methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/16/2012 00:13'!
actionToPerform

	^ textModel actionToPerformHolder content! !

!MethodBrowser methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/16/2012 00:12'!
actionToPerform: aBlock

	textModel actionToPerformHolder contents: aBlock! !

!MethodBrowser methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 1/18/2012 02:15'!
initialize
	"Initialization code for MethodBrowser"

	super initialize.

	listModel := self instantiateA: ListComposableModel.
	textModel := self instantiateA: TextModel.
	toolbarModel := self instantiateA: MethodToolbar.
	
	self focusOrder
		add: listModel;
		add: toolbarModel;
		add: textModel.
	
	listModel whenSelectedItemChanges: [:selection |
		(selection
			ifNil: [
				textModel text: ''.
				textModel behavior: nil.
				toolbarModel method: nil ]
			ifNotNil: [:m | 
				textModel text: m sourceCode.
				textModel behavior: m methodClass.
				toolbarModel method: m ])].
	
	self actionToPerform: [:t | self listModel selectedItem inspect ].
	textModel aboutToStyle: true.
	! !

!MethodBrowser methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/15/2012 22:31'!
listHolder

	^ listModel listHolder! !

!MethodBrowser methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/15/2012 22:31'!
listHolder: anHolder
	
	listModel listHolder: anHolder! !

!MethodBrowser methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 12/14/2011 14:51'!
listModel

	^ listModel! !

!MethodBrowser methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 1/15/2012 23:48'!
methods: aList

	" this code is equivalnet to
		self listModel items: aList
	  but the code above hide the fact the current entry point is th listModel listHolder in fact"
	
	self listHolder contents: aList! !

!MethodBrowser methodsFor: 'focus' stamp: 'BenjaminVanRyseghem 1/17/2012 03:16'!
takeKeyboardFocus

	^ self listModel takeKeyboardFocus! !

!MethodBrowser methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 12/14/2011 14:51'!
textModel

	^ textModel! !

!MethodBrowser methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/18/2012 01:58'!
toolbarModel

	^ toolbarModel! !

!MethodBrowser methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 1/16/2012 23:02'!
wrapWith: aBlock

	listModel wrappingBlock: aBlock! !

ComposableModel subclass: #MethodToolbar
	instanceVariableNames: 'methodHolder browseModel sendersModel versionModel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Examples'!

!MethodToolbar commentStamp: '<historical>' prior: 0!
A MethodToolbar is xxxxxxxxx.

b := MethodToolbar new.
b openWithSpec.
b method: (ButtonModel>>#state:).
b method: nil!

!MethodToolbar class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 1/18/2012 01:14'!
internSpec

	^ { #PanelMorph.
			#changeTableLayout.
			#listDirection:.		#rightToLeft.
			#addMorph:. {#model.	#browseModel.	#internSpec}.
			#addMorph:. {#model.	#sendersModel.	#internSpec}.
			#addMorph:. {#model.	#versionModel.	#internSpec}.
			#hResizing:.	#shrinkWrap.
			#vResizing:.	#shrinkWrap.	}! !

!MethodToolbar class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 1/18/2012 01:04'!
title

	^ 'Toolbar'! !

!MethodToolbar methodsFor: 'private' stamp: 'BenjaminVanRyseghem 1/18/2012 01:22'!
browseMethod

	methodHolder contents
		ifNotNil: [:aMethod | aMethod browse ]! !

!MethodToolbar methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/18/2012 01:17'!
browseModel

	^ browseModel! !

!MethodToolbar methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 1/18/2012 01:48'!
initialize
	"Initialization code for MethodToolbar"

	super initialize.

	methodHolder := nil asValueHolder.
	browseModel := self instantiateA: ButtonModel.
	sendersModel := self instantiateA: ButtonModel.
	versionModel := self instantiateA: ButtonModel.

	self setFocusOrder.
	
	self setBrowseModel.
	self setVersionModel.
	self setSendersModel.

	self registerForChanges.! !

!MethodToolbar methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 1/18/2012 00:35'!
method: aMethod

	methodHolder contents: aMethod! !

!MethodToolbar methodsFor: 'display' stamp: 'BenjaminVanRyseghem 1/18/2012 01:07'!
openWithSpec
	| window |

	window := StandardWindow new
					addMorph: self buildWithSpec
					fullFrame: (LayoutFrame 
						fractions: (0@0 corner: 1@0) 
						offsets: (0@0 corner: 0@30)).
	window openInWorld.
	window title: self title.
	^ window! !

!MethodToolbar methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 1/18/2012 01:04'!
registerForChanges

	methodHolder whenChangedDo: [:contents || boolean |
		boolean := contents
						ifNil: [ false ]
						ifNotNil: [ true ].
		browseModel state: boolean.
		sendersModel state: boolean.
		versionModel state: boolean.]! !

!MethodToolbar methodsFor: 'private' stamp: 'BenjaminVanRyseghem 1/18/2012 01:21'!
sendersMethod

	methodHolder contents
		ifNotNil: [:aMethod | 
			| strm array |
			strm := (array := Array new: {} size + 1) writeStream.
			strm nextPut: nil.
			strm nextPutAll: {}.

			aMethod ifNil: [ | selector |
				selector := UIManager default request: 'Type selector:' initialAnswer: 'flag:'.
				selector ifNil: [selector := String new].
				selector := selector copyWithout: Character space.
				^ selector isEmptyOrNil ifFalse: [
					(Symbol hasInterned: selector
						ifTrue: [ :aSymbol |
							array at: 1 put: aSymbol.
							self systemNavigation perform: #browseAllCallsOn: withArguments: array])
						ifFalse: [ self inform: 'no such selector']
				]
			].

			array at: 1 put: aMethod selector.
			self systemNavigation perform: #browseAllCallsOn: withArguments: array]! !

!MethodToolbar methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/18/2012 01:17'!
sendersModel

	^ sendersModel! !

!MethodToolbar methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 1/18/2012 01:13'!
setBrowseModel
	
	browseModel
		state: false;
		label: 'Browse';
		action: [ self browseMethod ].
	
! !

!MethodToolbar methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 1/18/2012 00:36'!
setFocusOrder

	self focusOrder
		add: browseModel;
		add: sendersModel;
		add: versionModel.
		! !

!MethodToolbar methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 1/18/2012 01:18'!
setSendersModel
	
	sendersModel
		state: false;
		label: 'Senders';
		action: [ self sendersMethod ].
	
! !

!MethodToolbar methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 1/18/2012 01:13'!
setVersionModel
	
	versionModel
		state: false;
		label: 'Version';
		action: [ self versionMethod ].
	
! !

!MethodToolbar methodsFor: 'private' stamp: 'BenjaminVanRyseghem 1/18/2012 01:22'!
versionMethod

	methodHolder contents
		ifNotNil: [:aMethod | 
			| selector class | 
	(selector := aMethod selector)
		ifNil: [ self inform: 'Sorry, only actual methods have retrievable versions.'. ^nil ]
		ifNotNil: [
			class := aMethod methodClass.
			(VersionsBrowser
				browseVersionsOf: aMethod
				class: class
				meta: class isMeta
				category: aMethod category
				selector: selector)]]! !

!MethodToolbar methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/18/2012 01:17'!
versionModel

	^ versionModel! !

ComposableModel subclass: #SendersBrowser
	instanceVariableNames: 'listModel textModel listHolder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Browsers'!

!SendersBrowser commentStamp: '<historical>' prior: 0!
A SendersBrowser is a UI used to browse senders and implementors

Basically, it's composed of a list used to browse methods, and of a text pane used to see the source code of the selected method!

!SendersBrowser classSide methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 1/12/2012 19:22'!
internSpec

	^ {#PanelMorph.
			#changeTableLayout.
			#listDirection:.	#bottomToTop.
			#addMorph:.	{#model.	#listModel.		#internSpec}.
			#addMorph:.	{#model.	#textModel.	#internSpec}.}! !

!SendersBrowser classSide methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 1/12/2012 19:23'!
title

	^ 'Senders/Implementors'! !

!SendersBrowser methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 1/15/2012 22:29'!
initialize
	"Initialization code for SendersBrowser"
	| linkHolder behaviorHolder |

	super initialize.
	
	listHolder := {} asValueHolder.
	listModel := ListComposableModel new.
	listModel listHolder: listHolder.
	textModel := TextModel new.
	
	linkHolder := ComposableValueHolder on: listModel selectionHolder do: [:index :selection | 
		selection contents 
			ifNil: [ '' ] 
			ifNotNil: [:method | method sourceCode ]].
		
	behaviorHolder := ComposableValueHolder on: listModel selectionHolder do: [:index :selection | 
		selection contents 
			ifNil: [ nil ] 
			ifNotNil: [:method | method methodClass ]].
		
	textModel textHolder: linkHolder.
	textModel behaviorHolder: behaviorHolder.
	textModel aboutToStyleHolder contents: true.! !

!SendersBrowser methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/12/2012 19:23'!
listHolder

	^ listHolder! !

!SendersBrowser methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/12/2012 19:45'!
listHolder: anHolder

	listHolder := anHolder.
	self listModel listHolder: anHolder.! !

!SendersBrowser methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/12/2012 19:23'!
listModel

	^ listModel! !

!SendersBrowser methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/12/2012 19:23'!
textModel

	^ textModel! !

ComposableModel subclass: #TabClassical
	instanceVariableNames: 'container tab'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Examples'!

!TabClassical commentStamp: '<historical>' prior: 0!
A TabClassical is an example to learn how to use tabs ;)!

!TabClassical class methodsFor: 'instance creation' stamp: 'BenjaminVanRyseghem 1/5/2012 18:33'!
associations: associations

	^ self new
		addTabs: associations;
		yourself.! !

!TabClassical class methodsFor: 'example' stamp: 'BenjaminVanRyseghem 1/5/2012 18:39'!
example

	| tc testMorph model anotherMorph associations |
	
	model := ClassMethodBrowser new.
	
	testMorph := model buildWithSpec: #internSpec.
	anotherMorph := model buildWithSpec: #internSpec2.
	
	associations := {'Test' -> testMorph. 'Another'-> anotherMorph}.
	tc := self associations: associations.
	
	model listHolder contents: (Smalltalk allClasses).
	tc open.! !

!TabClassical methodsFor: 'display' stamp: 'BenjaminVanRyseghem 1/7/2012 05:59'!
addAll: window
	
	window
		addMorph: tab
		fullFrame: (LayoutFrame
				fractions: (0 @ 0 corner: 1 @ 0)
				offsets: (0 @ 0 corner: 0 @ 35)).
				
	window
		addMorph: container
		fullFrame: (LayoutFrame 
				fractions: (0 @ 0 corner: 1 @ 1)
				offsets: (0 @ 35 corner: 0 @ 0)).
				
	tab tabs size = 0 ifTrue: [ ^ self ].
	tab color: Color white darker darker! !

!TabClassical methodsFor: 'display' stamp: 'BenjaminVanRyseghem 1/15/2012 22:13'!
addTabs: associations

	associations do: [:each |
		self tab 
			addTab: (Text fromString: each key) 
			withAction: [ 	self setMorph: each value ].
		container submorphs isEmpty ifTrue: [ container addMorph: each value ]]! !

!TabClassical methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 1/5/2012 18:30'!
initialize
	"Initialization code for TabClassical"

	super initialize.

	container := PanelMorph new changeTableLayout; listDirection: #bottomToTop; color: Color white; yourself.
	tab := PluggableTabBarMorph on: self.
	tab width: 150.! !

!TabClassical methodsFor: 'display' stamp: 'BenjaminVanRyseghem 1/5/2012 18:21'!
open

	| window panel |
	window := StandardWindow new.
	panel := PanelMorph new hResizing: #spaceFill; vResizing: #spaceFill; changeProportionalLayout.

	window addMorph: panel frame: (0@0 corner: 1@1).
	self addAll: panel.
	window openInWorld.
	
	window title: 'Tab Test'! !

!TabClassical methodsFor: 'private' stamp: 'BenjaminVanRyseghem 1/3/2012 18:59'!
setMorph: aMorph

	container removeAllMorphs.
	container addMorph: aMorph.
	aMorph color: Color white.
	aMorph takeKeyboardFocus.! !

!TabClassical methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/3/2012 19:02'!
tab

	^ tab! !

ComposableModel subclass: #TestingComposableModel
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Tests'!

!TestingComposableModel commentStamp: '<historical>' prior: 0!
A TestingComposableModel is a stupid composable model used to test SpecInterpreter!

!TestingComposableModel class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 1/19/2012 16:43'!
internSpec

	^ {#List.
		#model:.	#model.}! !

!TestingComposableModel class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 1/19/2012 17:35'!
testingSpec

	^ {#Text.
		#model:.	#model.}! !

!TestingComposableModel class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 1/19/2012 16:43'!
title

	^ 'You should not see me !!'! !

ComposableModel subclass: #TextModel
	instanceVariableNames: 'aboutToStyleHolder behaviorHolder textHolder actionToPerformHolder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec'!

!TextModel commentStamp: '<historical>' prior: 0!
A TextComposableModel is an applicative model which handle a basic text.

| t |
t:= TextComposableModel new.
t openWithSpec.
t textHolder contents: 'This is a test'.

self example!

!TextModel class methodsFor: 'example' stamp: 'BenjaminVanRyseghem 12/15/2011 13:30'!
example
	| instance |
	
	instance := self new.
	instance openWithSpec.
	instance textHolder contents: 'This is a test'.! !

!TextModel class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 1/23/2012 19:14'!
internSpec

	^ {#PluggableTextMorph.
	    		#on:text:accept:readSelection:menu:. 	#model. #getText. #accept:. nil. nil.
			#hResizing:. 							#spaceFill.
			#vResizing:. 							#spaceFill.
			#eventHandler:.					{	#EventHandler. #on:send:to:. #keyStroke.	#keyStroke:fromMorph:. #model	}}! !

!TextModel class methodsFor: 'instance creation' stamp: 'BenjaminVanRyseghem 12/13/2011 23:34'!
textHolder: textHolder

	^ self new
		textHolder: textHolder;
		yourself! !

!TextModel class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 12/13/2011 23:31'!
title

	^ 'Text'! !

!TextModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 1/16/2012 22:59'!
aboutToStyle: aBoolean	

	aboutToStyleHolder contents: [ aBoolean ]! !

!TextModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 1/16/2012 22:58'!
aboutToStyleBlock: aBlockReturningABoolean	

	aboutToStyleHolder contents: aBlockReturningABoolean! !

!TextModel methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/16/2012 00:06'!
aboutToStyleHolder

	^ aboutToStyleHolder! !

!TextModel methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/16/2012 00:06'!
aboutToStyleHolder: aValueHolder

	aboutToStyleHolder := aValueHolder.
	aValueHolder whenChangedDo: [ self changed: #getText ].
	self changed: #getText.! !

!TextModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 1/16/2012 00:10'!
accept: aText

	self textHolder contents: aText.
	self actionToPerformHolder contents value: aText.
	self changed: #clearUserEdits.! !

!TextModel methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/16/2012 00:08'!
actionToPerformHolder

	^ actionToPerformHolder! !

!TextModel methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/16/2012 00:10'!
actionToPerformHolder: aValueHolder

	actionToPerformHolder := aValueHolder.! !

!TextModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 1/16/2012 02:15'!
behavior: aText

	behaviorHolder contents: aText! !

!TextModel methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 12/15/2011 13:41'!
behaviorHolder

	^ behaviorHolder! !

!TextModel methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/3/2012 17:11'!
behaviorHolder: aValueHolder
	
	behaviorHolder := aValueHolder.
	aValueHolder whenChangedDo: [ 
		self morph ifNotNil: [:mrph | mrph classOrMetaClass: aValueHolder contents ].
		self changed: #getText ].
	self morph ifNotNil: [:mrph | mrph classOrMetaClass: aValueHolder contents ].
	self changed: #getText! !

!TextModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 12/14/2011 14:17'!
doItContext

	^ nil! !

!TextModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 12/14/2011 14:17'!
doItReceiver

	^ self selectedBehavior
		ifNil: [ nil ]
		ifNotNil: [:class | class theNonMetaClass ]! !

!TextModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 12/14/2011 17:26'!
getText
	
	^ self textHolder contents! !

!TextModel methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 1/16/2012 00:07'!
initialize
	"Initialization code for TextComposableModel"

	super initialize.

	self textHolder: '' asValueHolder.
	self behaviorHolder: nil asValueHolder.
	self aboutToStyleHolder: [ false ] asValueHolder.
	actionToPerformHolder := [:text | ] asValueHolder.! !

!TextModel methodsFor: 'focus' stamp: 'BenjaminVanRyseghem 1/23/2012 19:14'!
isEventForNextFocus: aKeyboardEvent

	^ aKeyboardEvent keyString = '<Ctrl-Cmd-right>'! !

!TextModel methodsFor: 'focus' stamp: 'BenjaminVanRyseghem 1/23/2012 19:08'!
isEventForPreviousFocus: aKeyboardEvent

	^ aKeyboardEvent keyString = '<Ctrl-Cmd-left>'! !

!TextModel methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 12/14/2011 14:09'!
okToStyle

	^ true! !

!TextModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 12/14/2011 14:14'!
selectedBehavior

	^ self behaviorHolder contents! !

!TextModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 12/14/2011 14:21'!
shoutAboutToStyle: aMorph
	
	^ self aboutToStyleHolder contents value! !

!TextModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 1/15/2012 23:26'!
text: aText

	textHolder contents: aText! !

!TextModel methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 12/15/2011 13:41'!
textHolder

	^ textHolder! !

!TextModel methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 12/15/2011 13:41'!
textHolder: anObject
	
	textHolder := anObject.
	anObject whenChangedDo: [ self changed: #getText ].
	self changed: #getText.! !

Object subclass: #MorphWrapper
	instanceVariableNames: 'layout morph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec'!

!MorphWrapper commentStamp: '<historical>' prior: 0!
A MorphWrapper is used to wrap a morph with a layout in the goal to be easily added to another morph!

!MorphWrapper class methodsFor: 'instance creation' stamp: 'BenjaminVanRyseghem 11/14/2011 16:27'!
morph: morph layout: layout 

	^ self new
		layout: layout;
		morph: morph;
		yourself! !

!MorphWrapper methodsFor: 'adding' stamp: 'BenjaminVanRyseghem 11/14/2011 16:31'!
addIn: aContainer

	aContainer
		addMorph: morph
		fullFrame: layout! !

!MorphWrapper methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/14/2011 16:38'!
frame: relFrame
	| frame |
	frame := LayoutFrame new.
	frame 
		leftFraction: relFrame left; 
		rightFraction: relFrame right; 
		topFraction: relFrame top; 
		bottomFraction: relFrame bottom.
	layout := frame! !

!MorphWrapper methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/14/2011 16:38'!
fullFrame: anObject
	
	layout := anObject! !

!MorphWrapper methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/14/2011 16:27'!
morph: anObject
	
	morph := anObject! !

Object subclass: #NewValueHolder
	instanceVariableNames: 'announcer contents'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec'!

!NewValueHolder commentStamp: '<historical>' prior: 0!
A NewValueHolder is a new implementation of ValueHolder based on Announcements!

NewValueHolder subclass: #ComposableValueHolder
	instanceVariableNames: 'block originalValueHolder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec'!

!ComposableValueHolder commentStamp: '<historical>' prior: 0!
A ComposableValueHolder is a value holder based on another value holder which content is calculated regarding the original value holder content!

!ComposableValueHolder class methodsFor: 'instance creation' stamp: 'BenjaminVanRyseghem 11/7/2011 18:06'!
on: originalValueHolder do: block

	^ self new
		block: block;
		originalValueHolder: originalValueHolder;
		yourself! !

!ComposableValueHolder methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/7/2011 18:06'!
block

	^ block! !

!ComposableValueHolder methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/7/2011 18:06'!
block: anObject
	
	block := anObject! !

!ComposableValueHolder methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/7/2011 18:06'!
originalValueHolder

	^ originalValueHolder! !

!ComposableValueHolder methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/28/2011 17:27'!
originalValueHolder: aValueHolder
	
	originalValueHolder := aValueHolder.
	aValueHolder whenChangedSend: #updateContents to: self.
	self update: #update! !

!ComposableValueHolder methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/28/2011 09:57'!
updateContents

	| args |
	args := self originalValueHolder contents.
	args isArray
		ifTrue: [ self contents: (self block valueWithArguments: args)]
		ifFalse: [ self contents: (self block value: args)].! !

!NewValueHolder class methodsFor: 'instance creation' stamp: 'BenjaminVanRyseghem 11/28/2011 09:28'!
contents: contents

	^ self new
		contents: contents;
		yourself! !

!NewValueHolder methodsFor: 'announcements' stamp: 'BenjaminVanRyseghem 11/28/2011 09:31'!
addDependent: aDependent

	self halt: 'Former API, should be changed'.! !

!NewValueHolder methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/28/2011 09:23'!
contents

	^ contents! !

!NewValueHolder methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/28/2011 09:35'!
contents: anObject
	
	| oldValue |
	oldValue := contents.
	contents := anObject.
	self contentsChanged: oldValue.! !

!NewValueHolder methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/28/2011 17:24'!
contentsChanged
	
	announcer announce: (ValueChanged newContents: contents)! !

!NewValueHolder methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/28/2011 17:24'!
contentsChanged: oldValue
	
	announcer announce: (ValueChanged oldContents: oldValue newContents: contents)! !

!NewValueHolder methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/16/2012 01:10'!
initialize

	super initialize.
	
	announcer := Announcer new! !

!NewValueHolder methodsFor: 'announcements' stamp: 'BenjaminVanRyseghem 1/16/2012 01:53'!
whenChangedDo: aBlock

	| block |
	block := [ aBlock cull: self contents ].
	announcer on: ValueChanged do: block! !

!NewValueHolder methodsFor: 'announcements' stamp: 'BenjaminVanRyseghem 11/28/2011 17:27'!
whenChangedSend: aSelector to: aReceiver

	announcer on: ValueChanged send: aSelector to: aReceiver! !

NewValueHolder subclass: #SelectionValueHolder
	instanceVariableNames: 'index selection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec'!

!SelectionValueHolder commentStamp: '<historical>' prior: 0!
A SelectionValueHolder is a value holder especially designed to contains a list selection (an index + an object)!

!SelectionValueHolder class methodsFor: 'as yet unclassified' stamp: 'BenjaminVanRyseghem 11/7/2011 21:00'!
index: indexValueHolder selection: selectionValueHolder

	^ self new
		index: indexValueHolder;
		selection: selectionValueHolder;
		yourself! !

!SelectionValueHolder methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/17/2011 20:09'!
contents

	^ {index. selection}! !

!SelectionValueHolder methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/7/2011 20:57'!
index

	^ index! !

!SelectionValueHolder methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/7/2011 20:59'!
index: anIndex

	index := anIndex.
	self changed: #index! !

!SelectionValueHolder methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/28/2011 17:27'!
initialize

	super initialize.
	index := 0 asValueHolder.
	selection := nil asValueHolder.
	
	index whenChangedSend: #contentsChanged to: self.
	selection whenChangedSend: #contentsChanged to: self.! !

!SelectionValueHolder methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 1/7/2012 06:48'!
reset

	self index contents: 0.
	self selection contents: nil! !

!SelectionValueHolder methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/7/2011 20:57'!
selection

	^ selection! !

!SelectionValueHolder methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/7/2011 20:58'!
selection: aSelection

	selection := aSelection.
	self changed: #selection! !

!Object methodsFor: '*Spec' stamp: 'BenjaminVanRyseghem 11/28/2011 09:29'!
asValueHolder

	^ NewValueHolder contents: self! !

Object subclass: #SpecInterpreter
	instanceVariableNames: 'instance'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec'!

SpecInterpreter class
	instanceVariableNames: 'Bindings'!

!SpecInterpreter commentStamp: '<historical>' prior: 0!
#model is a magic world to swap the symbol to the model object
!

SpecInterpreter class
	instanceVariableNames: 'Bindings'!

!SpecInterpreter class methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/19/2012 16:33'!
bindings

	^ Bindings ifNil: [ self initializeBindings. ^ Bindings ]! !

!SpecInterpreter class methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 1/19/2012 16:39'!
buildMorphFor: aComposableModel
	
	^ self new buildMorphFor: aComposableModel! !

!SpecInterpreter class methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 1/19/2012 16:40'!
buildMorphFor: aComposableModel withSpec: aSymbol
	
	^ self new buildMorphFor: aComposableModel withSpec: aSymbol! !

!SpecInterpreter class methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 1/19/2012 16:40'!
buildWindowFor: aComposableModel
	
	^ self new buildWindowFor: aComposableModel! !

!SpecInterpreter class methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 1/19/2012 16:37'!
initializeBindings

	Bindings := Dictionary new.

	Bindings
		at: #List put: #PluggableListMorph;
		at: #Text put: #PluggableTextMorph;
		at: #Button put: #PluggableButtonMorph.! !

!SpecInterpreter class methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 1/19/2012 16:36'!
interpretASpec: aSpec model: aModel
	
	^ self new	interpretASpec: aSpec model: aModel! !

!SpecInterpreter class methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/19/2012 16:34'!
resetBindings

	Bindings := nil! !

!SpecInterpreter methodsFor: 'building' stamp: 'BenjaminVanRyseghem 1/19/2012 17:19'!
buildMorphFor: aComposableModel
	
	^ self buildMorphFor: aComposableModel withSpec: #internSpec! !

!SpecInterpreter methodsFor: 'building' stamp: 'BenjaminVanRyseghem 1/19/2012 17:19'!
buildMorphFor: aComposableModel withSpec: aSymbol
	
	^ self interpretASpec: (aComposableModel class perform: aSymbol) model: aComposableModel! !

!SpecInterpreter methodsFor: 'building' stamp: 'BenjaminVanRyseghem 1/19/2012 16:38'!
buildWindowFor: aComposableModel
	
	^ self interpretASpec: aComposableModel class windowSpec model: aComposableModel! !

!SpecInterpreter methodsFor: 'interpreting' stamp: 'BenjaminVanRyseghem 1/19/2012 16:31'!
classKeys

	^ Smalltalk keys union: self class bindings keys! !

!SpecInterpreter methodsFor: 'interpreting' stamp: 'BenjaminVanRyseghem 1/19/2012 16:31'!
convertSymbolOfClassToInstance: aSymbol

	| symbol |
	symbol := self class bindings at: aSymbol ifAbsent: [ aSymbol ].

	^ (Smalltalk at: symbol) new! !

!SpecInterpreter methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/19/2012 16:25'!
instance

	^ instance! !

!SpecInterpreter methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/19/2012 16:25'!
instance: anObject
	
	instance := anObject! !

!SpecInterpreter methodsFor: 'interpreting' stamp: 'BenjaminVanRyseghem 1/19/2012 17:39'!
interpretASpec: aSpec model: aModel
	| index |

	aSpec ifNil: [ ^ nil ].
	(aSpec isCollection and: [aSpec isSequenceable]) ifFalse: [ ^ aSpec ].
	aSpec isString ifTrue: [ 
			^ aSpec == #model
				ifTrue: [ aModel ]
				ifFalse: [ 
					(self classKeys identityIncludes: aSpec)
						ifTrue: [ self convertSymbolOfClassToInstance: aSpec ]
						ifFalse: [ aSpec ]]].

	" aSpec is an Array "
	instance := self class interpretASpec: aSpec first model: aModel.
	index := 2.	
	
	[ index <= aSpec size ]
		whileTrue: [ | selector numArgs args newInstance result |
			selector := aSpec at: index.
			numArgs := selector numArgs.
			args := aSpec copyFrom: index + 1 to: index + numArgs.
			args := args collect: [:each | self class interpretASpec: each model: aModel].
			index := index + numArgs + 1.
			newInstance := instance perform: selector withArguments: args.
			index > aSpec size
				ifTrue: [
					aModel morph: instance.
					^ self interpretASpec: newInstance model: instance ].
			instance := newInstance].
	"If you get here, there is a problem. Must probably it comes from the Spec which is wrongly defined"
	self assert: false.
	^ instance! !
